/*
	Copyright (c) 2002 Jens Tinz
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// This file contains a collection of functions for use with the pie menu.
// They're kept in a hashtable to avoid naming conflicts.
// Please contact me if you have useful new functions or can improve 
// on the existing ones.


var pie_f= {
	// navigation
	stop : function()				{	BrowserStop();																},
	reload : function(skip)		{	if(skip) BrowserReloadSkipCache(); else BrowserReload();						},
	next : function()				{	BrowserForward();															},
	prev : function()				{	BrowserBack();																},
	up : function(url)			{
		var page= url.substring(url.substring(0, url.length-1).lastIndexOf('/')+1, url.length);
		alert(page);
		url= url.substring(0, url.substring(0, url.length-1).lastIndexOf('/')+1);
		loadURI(url);	
	},
	
	// tasks
	openMessenger : function()	{	toMessengerWindow(); },
	openAddressbook : function()	{	toAddressBook(); },
	openChat : function()		{	toIRC(); },
	openComposer : function()	{	toEditor(); },

	// windows
	openWindow : function(url)	{	openNewWindowWith(url);													},
	closeWindow : function()		{	BrowserCloseWindow();														},
	toggleFullscreen : function()	{	BrowserFullScreen();															},
	minimizeWindow : function()	{	window.minimize();															},
	newWindow : function(duplicate)	{	
		var url= "";
		if(duplicate) {
			url= window._content.document.location;
		} else {
			var prefs= Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("");
			if(prefs.getIntPref("browser.startup.page")== 1)
				url= getHomePage();
		}
		window.open(url); 
	},

	// frames
	viewFrameInfo : function(url)	{	BrowserPageInfo(url);															},
	reloadFrame : function(framedoc) {	framedoc.location.reload();												},
	
	// edit
	undo : function()				{	goDoCommand('cmd_undo');													},
	cut : function()				{	goDoCommand('cmd_cut');														},
	copy : function()				{	goDoCommand('cmd_copy');													},
	paste : function()			{	goDoCommand('cmd_paste');													},

	// document
	viewInfo : function()			{	BrowserPageInfo();															},
	saveAs : function(url, name)	{	saveURL(url, name, null, false);											},
	savePage : function(doc)		{	saveDocument(doc);															},
	viewSource : function(doc)	{	
		if(getSelection()!= "") {
			try { 
				gContextMenu.viewPartialSource('selection');
			} catch(e) {
				BrowserViewSourceOfDocument(doc);
			}
		} else BrowserViewSourceOfDocument(doc);
	},
	bookmark : function(url, name, edit) {	
		if(!edit) {
			BookmarksUtils.addBookmark(url, name, url, null, true);
		} else {
			if(getBrowser().browsers.length> 1)
				BookmarksUtils.addBookmarkForTabBrowser(gBrowser);
			else BookmarksUtils.addBookmarkForBrowser(gBrowser.webNavigation, true);
		}
	},
	scroll : function(doc, dir) {
		var dx= doc.body.scrollWidth/ 160;
		if(dx< 10) dx= 10; if(dx> 50) dx= 50;
		dx= dir=='L'? -dx: (dir=='R'? dx: 0);
		
		var dy= doc.body.scrollHeight/ 160;
		if(dy< 10) dy= 10; if(dy> 50) dy= 50;
		dy= dir=='U'? -dy: (dir=='D'? dy: 0);
		
		window._content.scrollBy(dx, dy);
		pie_act.x+= dx; pie_act.y+= dy;
	},

	// selection 
	searchWeb : function(string, newwin) {
		if(newwin) BrowserOpenTab();
		loadURI('http://www.google.com/search?q='+ string); 
	},
	loadUrl : function(string, newwin) {	
		if(newwin) BrowserOpenTab();
		loadURI(string);
	},
	translate : function(string, newwin) {
		if(newwin) BrowserOpenTab();
		loadURI('http://babelfish.altavista.com/'); 
		setTimeout("window._content.document.getElementsByTagName('textarea')[0].value= \""+string+ "\";", 3000);
	},

	// image
	toggleImageBlock : function(url)		{
		var pmanager= Components.classes["@mozilla.org/permissionmanager;1"]
			.getService(Components.interfaces.nsIPermissionManager);
		if(pmanager!= null) {
			var blocked= pmanager.testForBlocking(url, IMAGEPERMISSION);
			pmanager.add(url, blocked, IMAGEPERMISSION);
		}
	},
	
	// tabs
	newTab : function(duplicate)	{
		var oldloc= window._content.location; 
		BrowserOpenTab(); 
		if(duplicate) 
			window._content.document.location= oldloc; 
	},
	openTab : function(url) 		{	openNewTabWith(url);														},
	closeTab : function()			{	BrowserCloseTabOrWindow();													},
	nextTab : function() { //Contributed by Scurrie
		var tab, curTab, curTabNum, nextTab, l, i;
		if(getBrowser().mTabContainer.childNodes.length > 1) {
			l= getBrowser().mTabContainer.childNodes.length;
			curTab= getBrowser().selectedTab;
			for(i= 0; i < l; i++) {
				tab= getBrowser().mTabContainer.childNodes[i];
				if(tab == curTab) { curTabNum= i; }
			}
			if(curTabNum == getBrowser().mTabContainer.childNodes.length- 1) {
				nextTab= getBrowser().mTabContainer.childNodes[0];
			} else {
				nextTab= getBrowser().mTabContainer.childNodes[curTabNum+ 1]; 
			}
			getBrowser().selectedTab= nextTab;
		}
	},
	prevTab : function() { //Contributed by Scurrie
		var tab, curTab, curTabNum, nextTab, l, i;
		if(getBrowser().mTabContainer.childNodes.length> 1) {
			l= getBrowser().mTabContainer.childNodes.length;
			curTab= getBrowser().selectedTab;
			for(i= 0; i < l; i++) {
				tab= getBrowser().mTabContainer.childNodes[i];
				if (tab == curTab) { curTabNum = i; }
			}
			if(curTabNum == 0) {
				curTabNum= getBrowser().mTabContainer.childNodes.length- 1;
				nextTab= getBrowser().mTabContainer.childNodes[curTabNum];
			} else {
				nextTab= getBrowser().mTabContainer.childNodes[curTabNum- 1];
			}
			getBrowser().selectedTab= nextTab;
		}
	},

	// tool functions
	readClipboard : function() {
		var res= "";
	
		try {
			var clipb= Components.classes["@mozilla.org/widget/clipboard;1"].getService(Components.interfaces.nsIClipboard);
			var trans= Components.classes["@mozilla.org/widget/transferable;1"].createInstance(Components.interfaces.nsITransferable);
	
			trans.addDataFlavor("text/unicode");
			clipb.getData(trans, clipb.kSelectionClipboard);
	
			var data= {};
			var dataLen= {};
			trans.getTransferData("text/unicode", data, dataLen);
	
			if(data) {
				data= data.value.QueryInterface(Components.interfaces.nsISupportsWString);
				res= data.data.substring(0, dataLen.value/ 2);
			}
		} catch (ex) { }
		return res;
	},
	writeClipboard : function(string) {
		const cbhelper= Components.classes["@mozilla.org/widget/clipboardhelper;1"].getService(Components.interfaces.nsIClipboardHelper);
		cbhelper.copyString(string);
	}
}


