/*
	Copyright (c) 2002 Jens Tinz
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// This file manages the integration of the pie menu. 
// There's functionality for responding to events (mouse & prefs)
// and looking up the given context.

var pie_target= null;
var pie_target2= null;

var pie_doc= null;
var pie_frame_doc= null;
var pie_body= null;
var pie_alt= false;


// observes changes in the piePrefsPanel
function PiePrefsObserver() {
	try {
		// add the observer
		var mprefs= Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch(null);
		mprefs.QueryInterface(Components.interfaces.nsIPrefBranchInternal).addObserver(this.domain, this, false);
	} catch(ex) { }
}

PiePrefsObserver.prototype= {
	prefs : Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefService).getBranch("radialcontext."),
	domain : "radialcontext",
	
	// handle changes in settings
	observe : function(subject, topic, pname) {
		try {
			switch(pname) {
				case "radialcontext.use" :			setPieUse(this.prefs.getBoolPref("use"));							break;
				case "radialcontext.mbutton" :	pie_button= this.prefs.getIntPref("mbutton");						break;
				case "radialcontext.show_mask" :	pie_show_mask= this.prefs.getIntPref("show_mask");					break;
				case "radialcontext.suppr_mask" :	pie_suppr_mask= this.prefs.getIntPref("suppr_mask");				break;
				case "radialcontext.dragonly" :	PieMenu.prototype.dragonly= this.prefs.getBoolPref("dragonly");		break;
				case "radialcontext.ttip_delay" :	PieMenu.prototype.ttip_delay= this.prefs.getIntPref("ttip_delay");	break;
			}
		} catch(ex) { 
		}
	}
} 

// read the prefs, preload icons
function initPieIntegration(evt) {
	window.removeEventListener("load", initPieIntegration, true);
	initPieLayout();

	var use= true;
	try {
		use= pie_pobserver.prefs.getBoolPref("use");
		pie_button= pie_pobserver.prefs.getIntPref("mbutton");
		pie_show_mask= pie_pobserver.prefs.getIntPref("show_mask");
		pie_suppr_mask= pie_pobserver.prefs.getIntPref("suppr_mask");
		PieMenu.prototype.dragonly= pie_pobserver.prefs.getBoolPref("dragonly");
		PieMenu.prototype.ttip_delay= pie_pobserver.prefs.getIntPref("ttip_delay");
	} catch(ex) {
		pie_pobserver.prefs.setBoolPref("use", true);
		pie_pobserver.prefs.setBoolPref("dragonly", false);
		pie_pobserver.prefs.setIntPref("ttip_delay", 800);
		if(navigator.userAgent.toLowerCase().indexOf("mac")==- 1) {
			pie_pobserver.prefs.setIntPref("mbutton", 2);
			pie_pobserver.prefs.setIntPref("show_mask", 0);
			pie_pobserver.prefs.setIntPref("suppr_mask", 1);
		} else {
			// mac users need different defaults
			pie_pobserver.prefs.setIntPref("mbutton", 0);
			pie_pobserver.prefs.setIntPref("show_mask", 2);
			pie_pobserver.prefs.setIntPref("suppr_mask", 0);
		}
	}
	// activate radial menu
	if(use) {
		setPieUse(true);
		//preloadPieIcons();
	}
	// check if we're the only window -> starting up
	var wenum= Components.classes["@mozilla.org/embedcomp/window-watcher;1"]
		.getService(Components.interfaces.nsIWindowWatcher).getWindowEnumerator();
	wenum.getNext();
	if(!wenum.hasMoreElements()  && window.screenX!= 3) {
		// show tip dialog
		var tip_nr= 0;
		try {
			//pie_pobserver.prefs.setIntPref("tip_nr", 0);
			tip_nr= pie_pobserver.prefs.getIntPref("tip_nr");
		} catch(ex) { }
		if(tip_nr>= 0) 
			window.openDialog('chrome://radialcontext/content/pieTipDialog.xul', 'rc_tip', 'chrome,alwaysRaised,centerscreen,width=300,height=200');
	}
}

// adds or removes mouse listeners
function setPieUse(use) {
	var cm= document.getElementById("contentAreaContextMenu");
	if(use) {
		getBrowser().addEventListener("mousedown", handleMousedown, true);
		getBrowser().addEventListener("mouseup", handleMouseup, true);
		getBrowser().addEventListener("keydown", handleKeys, true);
		getBrowser().addEventListener("keyup", handleKeys, true);
		if(cm) cm.addEventListener("popupshowing", handlePopup, false);
	} else {
		getBrowser().removeEventListener("mousedown", handleMousedown, true);
		getBrowser().removeEventListener("mouseup", handleMouseup, true);
		getBrowser().removeEventListener("keydown", handleKeys, true);
		getBrowser().removeEventListener("keyup", handleKeys, true);
		if(cm) cm.removeEventListener("popupshowing", handlePopup, false);
	}
}

// suppress normal context menu
function handlePopup(evt) {
	if(pie_block_cmenu)
		evt.preventDefault();
	pie_block_cmenu= false;
}

// change menu labels
function handleKeys(evt) {
	if(evt.which== 18 && pie_act!= null)
		pie_act.updateLabels(evt.type== "keydown");
}

// hide the menu, clean up
function handleMouseup(evt)	{
	if(pie_state== 0) {
		pie_block_cmenu= false;
		window.removeEventListener("mousemove", handleMousemove, true);
	} else if(pie_state== 1) {
		if(evt.button== pie_button)
			PieMenu.prototype.r_action= PieMenu.prototype.r; 
			pie_state= 3;
	} else {
		pie_alt= evt.altKey;
		pie_alt|= pie_state== 2 && evt.button!= pie_button;
		window.removeEventListener("mousemove", handleMousemove, true);
		pie_act.hide();
	}
}

// let the menu update
function handleMousemove(evt)	{
	if(evt.originalTarget.ownerDocument!= pie_frame_doc)
		return;
	
	if(pie_state== 0) {
		if(Math.abs(evt.pageX- pie_xdown) + Math.abs(evt.pageY- pie_ydown)> 6)
			showPieMenu(evt.pageX, evt.pageY);
	} else {
		pie_act.update(evt.pageX, evt.pageY);
	}
}

// identify context and display menu
function handleMousedown(evt)	{
	var mask= (evt.ctrlKey? 1: 0) + (evt.shiftKey? 2: 0);
	pie_block_cmenu= true;
	
	// check whether menu should show up
	if(pie_state> 0) {
		evt.preventDefault();
		return;
	}
	var is_xul= false;
	for(var node= evt.originalTarget; node!= null; node= node.parentNode)
		if(node.toString().indexOf("XUL")>= 0)
			is_xul= true;
	if(is_xul
		|| evt.button!= pie_button 
		|| (pie_show_mask& mask)!= pie_show_mask 
		|| ((pie_suppr_mask& mask)== pie_suppr_mask && pie_suppr_mask!= 0)
	) {
		pie_block_cmenu= false;
		return;
	}

	// identify primary and secondary context, find body 
	var iname= "navigation";
	var iname2= "tasks";
	pie_target= this;
	pie_target2= null;
	pie_doc= window._content.document;
	pie_frame_doc= evt.originalTarget.ownerDocument;

	pie_body= pie_frame_doc.getElementsByTagName("BODY")[0];

	if((pie_doc.URL.indexOf("ftp")== 0 || pie_doc.URL.indexOf("file")== 0)) {
		iname= "folder";
	}
	if(getSelection()!= "") {
		iname2= "selection";
		pie_target2= getSelection();
	};
	for(var node= evt.originalTarget; node!= null; node= node.parentNode) {
		if(node.nodeType== Node.ELEMENT_NODE) {
			switch(node.localName.toUpperCase()) {
				case "AREA" :
				case "A" :
					if(node.href!= null && node.href!= "") {
						iname2= iname;
						pie_target2= pie_target;
						pie_target= node;		
						if(node.href.toUpperCase().indexOf("MAILTO")== 0)
							iname= "mail";	
						else iname= "link";	
					}
					break;
				case "IMG" :			
					iname2= iname;
					pie_target2= pie_target;
					pie_target= node;		
					iname= "image";		
					break;
				case "TEXTAREA" :	
					iname2= iname;
					pie_target2= pie_target;
					pie_target= node;		
					iname= "edit";		
					break;
				case "INPUT" :		
					if(node.type.toUpperCase()== "TEXT" || node.type.toUpperCase()== "PASSWORD") {
						iname2= iname;
						pie_target2= pie_target;
						pie_target= node;
						iname= "edit";
					}
					break;
			}
		}
	}
	pie_iname= iname;
	pie_iname2= iname2;
	
	pie_xdown= evt.pageX; 
	pie_ydown= evt.pageY; 
	window.addEventListener("mousemove", handleMousemove, true);
	if(!PieMenu.prototype.dragonly)
		showPieMenu(pie_xdown, pie_ydown);
	evt.preventDefault();
}

// show pie_act and modify for secondary context
function showPieMenu(mx, my) {
	pie_act= PieMenu.prototype.menus[pie_iname];
	if(pie_act== null)
		pie_act= new PieMenu(pie_iname);

	PieMenu.prototype.r_action= PieMenu.prototype.r_images; 
	pie_act.show(mx, my);
	pie_act.replaceItemData(4, PieMenu.prototype.itemsets[pie_iname2+ "_item"]);
	if(pie_doc!= pie_frame_doc) {
		pie_act.replaceItemData(2, PieMenu.prototype.itemsets["frame_item"]);
	} else pie_act.replaceItemData(2, PieMenu.prototype.itemsets["page_item"]);
	if(pie_iname== "navigation" || pie_iname== "folder") {
		if(getWebNavigation().canGoForward)
			pie_act.replaceItemData(1, PieMenu.prototype.itemsets["forward_item"]);
		else pie_act.replaceItemData(1, PieMenu.prototype.itemsets["forward_gray_item"]);

		if(getWebNavigation().canGoBack)
			pie_act.replaceItemData(5, PieMenu.prototype.itemsets["back_item"]);
		else pie_act.replaceItemData(5, PieMenu.prototype.itemsets["back_gray_item"]);
	}
}

// preload all icons in PieMenu.prototype.itemsets
function preloadPieIcons() {
	var img= new Image();
	var path= PieMenu.prototype.path;
	img.src= path+ "menu.png";
	img.src= path+ "submenu.png";
	img.src= path+ "labels.gif";
	for(var items in PieMenu.prototype.itemsets) {
		for(var i= 0; i< items.length; i++) {
			img.src= path+ items[i].src+ ".gif"
			img.src= path+ items[i].src+ "_a.gif"
		}
	}
}

// find text selection in current HTML document
function getSelection() {
	var focusedw= document.commandDispatcher.focusedWindow;
	var res= focusedw.__proto__.getSelection.call(focusedw);
	res= res.toString();
	res= res.replace( /^\s+/, "" );
	res= res.replace(/(\n|\r|\t)+/g, " ");
	res= res.replace(/\s+$/,"");
	return res;
}


var pie_iname;
var pie_iname2;

var pie_xdown= -1;				// needed for to give a tolerance to 'open when dragging'
var pie_ydown= -1;				// needed for to give a tolerance to 'open when dragging'
var pie_button= 2;				// mouse button that can open the menu
var pie_show_mask= 0;			// mask of modifiers for showing the menu
var pie_suppr_mask= 1;			// mask of modifiers for suppressing the menu
var pie_block_cmenu= false;		// whether the std context menu should be suppressed
var pie_pobserver= new PiePrefsObserver();
window.addEventListener("load", initPieIntegration, true);

