/*
	Copyright (c) 2002 Jens Tinz
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// This file defines the layout of the pie menu in different contexts.
// You can easily customize it. Helpful functions are provided in pieFunctions.js


function PieItem(type, label, altlabel, src, func) {
	this.type= type;										// -1 dummy, 0= item, 1= submenu (dummys are placeholders that don't create dead areas)
	this.label= label;										// wow - it's the label
	this.altlabel= altlabel;									// label when alt is down
	this.src=  src;										// name of icon source without suffix
	this.func= func;										// associated function or submenu 
	this.img= null;										// image object of icon
}

function initPieLayout() {
	var sb= document.getElementById("rcbundle");

	PieMenu.prototype.itemsets= {
		
		navigation : [
			new PieItem(1, sb.getString("tab"),				null,			"tab/menu",			"tab"),
			new PieItem(0, sb.getString("forward"),			null,			"navigation/forward",	"pie_f.next();"),
			new PieItem(1, sb.getString("page"),			null,			"page/menu",		"page"),
			new PieItem(0, sb.getString("reload"),			sb.getString("rlskip"),	"navigation/reload",	"pie_f.reload(pie_alt);"),
			new PieItem(0, sb.getString("context2"),		null,			"tasks/menu",		""),
			new PieItem(0, sb.getString("back"),			null,			"navigation/back",	"pie_f.prev();"),
			new PieItem(1, sb.getString("window"),			null,			"window/menu",		"window"),
			new PieItem(0, sb.getString("stop"),			null,			"navigation/stop",	"pie_f.stop();"),
		],
	
		folder : [
			new PieItem(1, sb.getString("tab"),				null,			"tab/menu",			"tab"),
			new PieItem(0, sb.getString("forward"),			null,			"navigation/forward",	"pie_f.next();"),
			new PieItem(1, sb.getString("page"),			null,			"page/menu",		"page"),
			new PieItem(0, sb.getString("reload"),			sb.getString("rlskip"),	"navigation/reload",	"pie_f.reload(pie_alt);"),
			new PieItem(0, sb.getString("context2"),		null,			"tasks/menu",		""),
			new PieItem(0, sb.getString("back"),			null,			"navigation/back",	"pie_f.prev();"),
			new PieItem(1, sb.getString("window"),			null,			"window/menu",		"window"),
			new PieItem(0, sb.getString("up"),				null,			"navigation/up",		"pie_f.up(pie_doc.URL);"),
		],
	
		link : [
			new PieItem(1, sb.getString("tab"),				null,			"tab/menu",			"tab"),
			new PieItem(0, sb.getString("innewtab"),		null,			"link/intab",			"pie_f.openTab(pie_target.href);"),
			new PieItem(1, sb.getString("page"),			null,			"page/menu",		"page"),
			new PieItem(0, sb.getString("savelink"),			null,			"link/save",			"pie_f.saveAs(pie_target.href, pie_target.innerHTML);"),
			new PieItem(0, sb.getString("context2"),		null,			"tasks/menu",		""),
			new PieItem(0, sb.getString("innewwin"),		null,			"link/inwin",			"pie_f.openWindow(pie_target.href);"),
			new PieItem(1, sb.getString("window"),			null,			"window/menu",		"window"),
			new PieItem(0, sb.getString("copyloc"),		null,			"link/copyloc",		"pie_f.writeClipboard(pie_target.href);"),
		],
	
		mail : [
			new PieItem(1, sb.getString("tab"),				null,			"tab/menu",			"tab"),
			new PieItem(-1, "",							null,			"empty",				null),
			new PieItem(1, sb.getString("page"),			null,			"page/menu",		"page"),
			new PieItem(-1, "",							null,			"empty",				null),
			new PieItem(0, sb.getString("context2"),		null,			"tasks/menu",		""),
			new PieItem(-1, "",							null,			"empty",				null),
			new PieItem(1, sb.getString("window"),			null,			"window/menu",		"window"),
			new PieItem(0, sb.getString("copyemail"),		null,			"link/email_address",	"pie_f.writeClipboard(pie_target.href.substring(7, pie_target.href.length));"),
		],
	
		edit : [
			new PieItem(1, sb.getString("tab"),				null,			"tab/menu",			"tab"),
			new PieItem(0, sb.getString("copy"),			null,			"edit/copy",			"pie_f.copy();"),
			new PieItem(1, sb.getString("page"),			null,			"page/menu",		"page"),
			new PieItem(0, sb.getString("paste"),			null,			"edit/paste",			"pie_f.paste();"),
			new PieItem(0, sb.getString("context2"),		null,			"tasks/menu",		""),
			new PieItem(0, sb.getString("cut"),				null,			"edit/cut",			"pie_f.cut();"),
			new PieItem(1, sb.getString("window"),			null,			"window/menu",		"window"),
			new PieItem(0, sb.getString("undo"),			null,			"edit/undo",			"pie_f.undo();"),
		],
		
		image : [
			new PieItem(1, sb.getString("tab"),				null,			"tab/menu",			"tab"),
			new PieItem(0, sb.getString("blockimg"),		null,			"image/block",		"pie_f.toggleImageBlock(pie_target.src);"),
			new PieItem(1, sb.getString("page"),			null,			"page/menu",		"page"),
			new PieItem(0, sb.getString("saveimg"),			null,			"image/save",		"pie_f.saveAs(pie_target.src, pie_doc.title);"),
			new PieItem(0, sb.getString("context2"),		null,			"tasks/menu",		""),
			new PieItem(0, sb.getString("viewimg"),			null,			"image/view",		"pie_f.loadUrl(pie_target.src);"),
			new PieItem(1, sb.getString("window"),			null,			"window/menu",		"window"),
			new PieItem(0, sb.getString("copyimgloc"),		null,			"image/copyloc",		"pie_f.writeClipboard(pie_target.src);"),
		],
		
		navigation_sub : [
			new PieItem(0, sb.getString("forward"),			null,			"navigation/forward",	"pie_f.next();"),
			new PieItem(0, sb.getString("reload"),			sb.getString("rlskip"),	"navigation/reload",	"pie_f.reload(pie_alt);"),
			new PieItem(0, sb.getString("back"),			null,			"navigation/back",	"pie_f.prev();"),
			new PieItem(0, sb.getString("stop"),			null,			"navigation/stop",	"pie_f.stop();"),
		],
	
		folder_sub : [
			new PieItem(0, sb.getString("forward"),			null,			"navigation/forward",	"pie_f.next();"),
			new PieItem(0, sb.getString("reload"),			sb.getString("rlskip"),	"navigation/reload",	"pie_f.reload(pie_alt);"),
			new PieItem(0, sb.getString("back"),			null,			"navigation/back",	"pie_f.prev();"),
			new PieItem(0, sb.getString("up"),				null,			"navigation/up",		"pie_f.up(pie_doc.URL);"),
		],
	
		image_sub : [
			new PieItem(1, sb.getString("navigation"),		null,			"navigation/menu_b",	"navigation_sub"),
			new PieItem(0, sb.getString("blockimg"),		null,			"image/block",		"pie_f.toggleImageBlock(pie_target2.src);"),
			new PieItem(-1, "",							null,			"empty",				null),
			new PieItem(0,  sb.getString("saveimg"),		null,			"image/save",		"pie_f.saveAs(pie_target2.src, pie_doc.title);"),
			new PieItem(-1, "",							null,			"empty",				null),
			new PieItem(0,  sb.getString("viewimg"),		null,			"image/view",		"pie_f.loadUrl(pie_target2.src);"),
			new PieItem(-1, "",							null,			"empty",				null),
			new PieItem(0,  sb.getString("copyimgloc"),		null,			"image/copyloc",		"pie_f.writeClipboard(pie_target2.src);"),
		],
	
		selection_sub : [
			new PieItem(0, sb.getString("open"),			sb.getString("aopen"), "selection/open",	"pie_f.loadUrl(pie_target2, pie_alt);"),
			new PieItem(0, sb.getString("translate"),		sb.getString("atranslate"), "selection/translate",	"pie_f.translate(pie_target2, pie_alt);"),
			new PieItem(0, sb.getString("searchweb"),		sb.getString("asearchweb"), "selection/searchweb","pie_f.searchWeb(pie_target2, pie_alt);"),
			new PieItem(0, sb.getString("copy"),			null,			"selection/copy",	"pie_f.writeClipboard(pie_target2);"),
		],
		
		tab : [
			new PieItem(0, sb.getString("nexttab"),			null,			"tab/next",			"pie_f.nextTab();"),
			new PieItem(0, sb.getString("closetab"),		null,			"tab/close",			"pie_f.closeTab();"),
			new PieItem(0, sb.getString("prevtab"),			null,			"tab/prev",			"pie_f.prevTab();"),
			new PieItem(0, sb.getString("newtab"),			sb.getString("duplicatetab"), "tab/new",			"pie_f.newTab(pie_alt);"),
		],
	
		window : [
			new PieItem(0, sb.getString("fullscreen"),		null,			"window/fullscreen",	"pie_f.toggleFullscreen();"),
			new PieItem(0, sb.getString("closewin"),		null,			"window/close",		"pie_f.closeWindow();"),
			new PieItem(0, sb.getString("minimize"),		null,			"window/minimize",	"pie_f.minimizeWindow();"),
			new PieItem(0, sb.getString("newwin"),			sb.getString("duplicatewin"), "window/new",		"pie_f.newWindow(pie_alt);"),
		],
		
		page : [
			new PieItem(0, sb.getString("bookmark"),		sb.getString("abookmark"),"page/bookmark",	"pie_f.bookmark(pie_doc.URL, pie_doc.title, pie_alt);"),
			new PieItem(0, sb.getString("savepage"),		null,			"page/save",			"pie_f.savePage(pie_doc);"),
			new PieItem(0, sb.getString("pagesource"),		null,			"page/source",		"pie_f.viewSource(pie_doc);"),
			new PieItem(0, sb.getString("pageinfo"),		null,			"page/info",			"pie_f.viewInfo();"),
		],
	
		frame : [
			new PieItem(-1, "",							null,			"empty",				null),
			new PieItem(0, sb.getString("viewframe"),		null,			"frame/view",		"pie_f.loadUrl(pie_frame_doc.URL);"),
			new PieItem(0, sb.getString("reloadframe"),		null,			"frame/reload",		"pie_f.reloadFrame(pie_frame_doc);"),
			new PieItem(0, sb.getString("saveframe"),		null,			"frame/save",		"pie_f.saveAs(pie_frame_doc.URL, pie_frame_doc.title);"),
			new PieItem(-1, "",							null,			"empty",				null),
			new PieItem(0,  sb.getString("framesource"),	null,			"frame/source",		"pie_f.viewSource(pie_frame_doc);"),
			new PieItem(1, sb.getString("page"),			null,			"page/menu_b",		"page"),
			new PieItem(0, sb.getString("frameinfo"),		null,			"frame/info",			"pie_f.viewFrameInfo(pie_frame_doc);"),
		],
	
		scroll : [
			new PieItem(2, sb.getString("rightscroll"),		null,			"scroll/right",		"pie_f.scroll(pie_doc, 'R');"),
			new PieItem(2, sb.getString("downscroll"),		null,			"scroll/down",		"pie_f.scroll(pie_doc, 'D');"),
			new PieItem(2, sb.getString("leftscroll"), 		null,			"scroll/left",			"pie_f.scroll(pie_doc, 'L');"),
			new PieItem(2, sb.getString("upscroll"), 		null,			"scroll/up",			"pie_f.scroll(pie_doc, 'U');"),
		],
	
		tasks : [
			new PieItem(0, sb.getString("mailnews"),		null,			"tasks/mail",			"pie_f.openMessenger();"),
			new PieItem(0, sb.getString("chatzilla"),		null,			"tasks/chat",			"pie_f.openChat();"),
			new PieItem(0, sb.getString("addressbook"),	null,			"tasks/addresses",	"pie_f.openAddressbook();"),
			new PieItem(0, sb.getString("composer"), 		null,			"tasks/composer",	"pie_f.openComposer();"),
		],
		
		context2 :			new PieItem(1, sb.getString("context2"),	null,	"tasks/menu",			"context2"),
		forward_item :		new PieItem(0, sb.getString("forward"),		null,	"navigation/forward",		"pie_f.next();"),
		forward_gray_item :	new PieItem(0, sb.getString("forward"),		null,	"navigation/forward_gray","pie_f.next();"),
		back_item :			new PieItem(0, sb.getString("back"),		null,	"navigation/back",		"pie_f.prev();"),
		back_gray_item :		new PieItem(0, sb.getString("back"),		null,	"navigation/back_gray",	"pie_f.prev();"),
		navigation_item :		new PieItem(1, sb.getString("navigation"),	null,	"navigation/menu",		"navigation_sub"),
		folder_item :			new PieItem(1, sb.getString("navigation"),	null,	"navigation/menu",		"folder_sub"),
		tasks_item :			new PieItem(1, sb.getString("tasks"),		null,	"tasks/menu",			"tasks"),
		selection_item :		new PieItem(1, sb.getString("selection"),	null,	"selection/menu",		"selection_sub"),
		image_item :			new PieItem(1, sb.getString("image"),		null,	"image/menu",			"image_sub"),
		frame_item :			new PieItem(1, sb.getString("frame"),		null,	"frame/menu",			"frame"),
		page_item :			new PieItem(1, sb.getString("page"),		null,	"page/menu",			"page"),
	}
}
