/*
	Copyright (c) 2002 Jens Tinz
	All rights reserved.

	Redistribution and use in source and binary forms, with or without
	modification, are permitted provided that the following conditions
	are met:
	1. Redistributions of source code must retain the above copyright
	notice, this list of conditions and the following disclaimer.
	2. Redistributions in binary form must reproduce the above copyright
	notice, this list of conditions and the following disclaimer in the
	documentation and/or other materials provided with the distribution.
	3. The name of the author may not be used to endorse or promote products
	derived from this software without specific prior written permission.

	THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
	IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
	OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
	IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
	INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
	NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
	DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
	THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
	(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
	THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/


// This file implements the actual pie menu.
// It can be attached to HTML documents and updated in response to events

var pie_act= null;				// menu currently displayed
var pie_state= 0;					// 0=not shown, 1=showing, 2=showing&mouse moved, 3=staying open

function PieMenu(itemsname) {
	this.x= 0;											// x coordinate of menu center
	this.y= 0;											// y coordinate of menu center
	this.xopen= 0;										// x coordinate of where menu was opened
	this.yopen= 0;										// x coordinate of where menu was opened
	this.sector= -1;										// index of item under mouse
	this.node= null;										// root of DOM tree containing the menu
	this.lnode= null;										// root of DOM tree containing the menu labels
	this.labelTrigger= null;								// trigger to present menu labels
	this.items= this.itemsets[itemsname];					// description of menu items

	PieMenu.prototype.menus[itemsname]= this;
}
	
PieMenu.prototype= {
	path :		"chrome://radialcontext/content/skin/",	// path to icons
	r :			44,										// outer radius
	r_hole :		17,										// inner radius
	r_images :	30,										// radius for middle of icons
	r_action :	30,										// active border
	icon_size :	22,
	ttip_delay : 900,										// tool tip delay
	
	menus :		new Array(1),							// array of all instances of PieMenu
	itemsets :	null,									// layout of menus
	dragonly :	false,									// show menu only when dragging?
	
	// return index of a given item
	indexOf : function(label) {
		for(var i=0; i<this.items.length; i++)
			if(label== this.items[i].label)
				return i;
		return -1;
	},
	
	// replace data of item at pos with the data of the given item
	replaceItemData : function(pos, item) {
		if(pos< 0 || pos> this.items.length)
			return;
	
		var ti= this.items[pos];
		ti.type= item.type;
		ti.label= item.label;
		ti.src=  item.src;
		ti.func= item.func;

		// update icon and label in the DOM
		if(ti.img!= null)
			ti.img.src= this.path+ item.src+ ".gif";
		if(this.lnode!= null)
			this.lnode.childNodes[pos+1].firstChild.nodeValue= ti.label;
	},
	
	// changes labels according to status of alt modifier
	updateLabels : function(altdown) {
		if(this.lnode!= null)
			for(var i=0; i<this.items.length; i++)
				this.lnode.childNodes[i+1].firstChild.nodeValue= (altdown && this.items[i].altlabel!= null)? this.items[i].altlabel: this.items[i].label;
	},
	
	// create DOM tree for the menu
	createNode : function() {
		// create a div to contain the items
		var node= pie_frame_doc.createElement("div");
		this.shieldCss(node);
		node.style.position= "absolute";
		node.style.width= 94;
		node.style.height= 94;
		node.style.zIndex= 99;
		node.style.lineHeight= "0px";
		node.style.textAlign= "left";
		
		// add the menu image
		var timg= pie_frame_doc.createElement("img");
		 this.shieldCss(timg);
		 timg.src= this.path+ "menu.png";
		 timg.style.width= 94;
		 timg.style.height= 94;
		 timg.alt="";
		node.appendChild(timg);
	
		// add and adjust icons
		var xofs=  this.r- this.icon_size/2+ 1 +1;
		var yofs= -this.r- this.icon_size/2- 5 +1;
		for(var i=0; i<this.items.length; i++) {
			var angle= -Math.PI/2 + (((i+1)* 2*Math.PI)/ this.items.length);
			var xpos= this.r_images* Math.cos(angle)+ xofs;
			var ypos= this.r_images* Math.sin(angle)+ yofs;
	
			timg= pie_frame_doc.createElement("img");
			 this.shieldCss(timg);
			 timg.style.position= "relative";
			 timg.style.display= "inline";
			 timg.style.left= Math.round(xpos)+"px";
			 timg.style.top= Math.round(ypos)+"px";
			 timg.style.width= this.icon_size;
			 timg.style.height= this.icon_size;
			 timg.src= this.path+ this.items[i].src+ ".gif";
			 timg.alt="";
			 this.items[i].img= timg;
			node.appendChild(timg);

			if(i==3 && this.items.length>4) {
				node.appendChild(pie_frame_doc.createElement("br"));
				yofs-= this.icon_size;
				xofs+= 3* this.icon_size;
			} else xofs-= this.icon_size;

		}
		this.node= node;
	},
	
	// attach DOM tree of the menu to pm_doc
	show : function(mx, my) {
		// create resources if necessary
		try {
			if(this.node== null)
				this.createNode();
	
			// move and unhide menu
			this.x= this.xopen= mx;
			this.y= this.yopen= my;
			pie_body.insertBefore(this.node, pie_body.childNodes[2]);
			this.node.style.left= mx- this.r- 1+ "px";
			this.node.style.top= my- this.r- 1+ "px";
			pie_state= 1;
		
			// set timeout for showing labels
			this.labelTrigger= setTimeout("pie_act.showLabels();", this.ttip_delay);
		} catch(ex) { }
	},
	
	// detach DOM of the menu from pm_doc and eval item.func
	hide : function() {
		// remove icon highlight
		var item= null;
		// remove menu
		try {
			if(this.sector>= 0) {
				item= this.items[this.sector];
				item.img.src= this.path+ item.src+ ".gif"
			}
			this.node.parentNode.removeChild(this.node);
			pie_state= 0;
		} catch(ex) { }
		this.hideLabels();
		clearTimeout(this.labelTrigger);
	
		// eval item
		if(this.sector>= 0) {
			this.sector= -1;
			if(item.type== 0 && item.func!= null) {
				try {
					eval(item.func);
				} catch(ex) { dump("CP: "+ ex.toString()+ "\n"); }
			}
		}
	},
	
	// handle dragging, rollovers, submenu display etc.
	update : function(mx, my) {
		// state change if was dragged
		if(pie_state== 1 && (
			Math.abs(mx- this.xopen)> 1 
			|| Math.abs(my- this.yopen)> 1
		)) pie_state= 2;

		// delay labels
		if(this.ttip_delay> 0) {
			clearTimeout(this.labelTrigger);
			this.labelTrigger= setTimeout("pie_act.showLabels();", this.ttip_delay);
		}
	
		// identify current sector
		var nsec= -1;
		var dst= Math.sqrt((mx-this.x)* (mx-this.x) + (my-this.y)* (my-this.y));  
		if(dst> this.r_hole) {
			var ang= Math.atan2(mx-this.x, this.y-my);
			if(ang< 0) ang+= 2* Math.PI;
			var tsec= ((this.items.length*ang)/ (2* Math.PI))- 1.5;
			if(tsec<= -1) tsec+= this.items.length;
			nsec= Math.ceil(tsec);
			// dummy elements musn't create dead areas
			if(this.items[nsec].type== -1) {
				if(nsec- tsec< .5)
					nsec++;
				else nsec--;
				if(nsec< 0) nsec= this.items.length-1;
				if(nsec>= this.items.length) nsec= 0;
			}
		}
		if(this.node!= null) {
			// rollover effect
			if(this.sector!= nsec) {
				if(this.sector>= 0)
					this.items[this.sector].img.src= this.path+ this.items[this.sector].src+ ".gif";
				if(nsec>= 0 && this.items[nsec].type== 0)
					this.items[nsec].img.src= this.path+ this.items[nsec].src+ "_a.gif";
			}
		}
		this.sector= nsec;
		var item= this.items[this.sector];
	
		if(dst> this.r_action) {
			// eval item
			if(item.type== 2 && item.func!= null)
				try {
					eval(item.func);
				} catch(ex) { dump("CP: "+ ex.toString()+ "\n"); }

			if(item.type!= 1) {
				// drag menu
				if(this.ttip_delay> 0)
					this.hideLabels();
				var fak= 1-(this.r_action/ dst);
				this.x+= (mx- this.x)* fak;
				this.y+= (my- this.y)* fak;
				if(this.node!= null) {
					this.node.style.left= this.x- this.r- 1+ "px";
					this.node.style.top= this.y- this.r- 1+ "px";
				}
				if(this.ttip_delay== 0 && this.lnode!= null) {
					this.lnode.style.left= this.x- 135+ "px";
					this.lnode.style.top= this.y- 55+ "px";
				}

			} else {
				// show submenu
				var state= pie_state;
				this.hide()
				var iname= item.func;
				pie_act= this.menus[iname];
				if(pie_act== null)
					pie_act= new PieMenu(iname);
				pie_act.show(mx, my);
				pie_state= state;
			}
		}
	},
	
	//-------functions for labels-------//

	// create DOM tree for the menu labels
	createLabelNode : function() {
		// create a div to contain the labels
		var node= pie_frame_doc.createElement("div");
		this.shieldCss(node);
		this.shieldTextCss(node);
		node.style.position= "absolute";
		node.style.height= "112px";
		node.style.width= "272px";
		node.style.zIndex= 98;
	
		var timg= pie_frame_doc.createElement("img");
		 this.shieldCss(timg);
		 timg.src= this.path+ "labels.gif";
		 timg.style.width= 272;
		 timg.style.height= 112;
		node.appendChild(timg);

		// add and adjust labels
		var icnt= this.items.length;
		for(var i=0; i< icnt; i++) {
			var xpos= (i>=icnt/2)? 8: 188;
			var ypos= ((i< icnt/2)? ((i+1)*8/icnt)*22: -((i+1)*8/icnt)*22+ 8*22)- i*22- 108;
			
			var tdiv= pie_frame_doc.createElement("div");
			 this.shieldCss(tdiv);
			 tdiv.style.position= "relative";
			 tdiv.style.left= xpos+"px";
			 tdiv.style.top= Math.round(ypos)+"px";
			 tdiv.style.height= "22px";
			 tdiv.style.width= "87px";
			 tdiv.appendChild(pie_frame_doc.createTextNode(this.items[i].label));
		node.appendChild(tdiv);
		}
		this.lnode= node;
	},
	
	// attach DOM tree of the menu labels to pm_doc
	showLabels : function() {
		// create resources if necessary
		if(this.lnode== null)
			this.createLabelNode();

		// compensate text zoom
		var zoom = ZoomManager.prototype.getInstance();
		var currentZoom = zoom.textZoom;
		this.lnode.style.lineHeight= 20*100/currentZoom+ "px";
		this.lnode.style.fontSize= 8*100/currentZoom+ "pt";
	
		try {
			// move and unhide labels
			pie_body.insertBefore(this.lnode, pie_body.childNodes[2]);
			this.lnode.style.left= this.x- 135+ "px";
			this.lnode.style.top= this.y- 55+ "px";
		} catch(ex) { }
	},
	
	// detach DOM tree of menu labels from pm_doc
	hideLabels : function() {
		try {
			this.lnode.parentNode.removeChild(this.lnode);
		} catch(ex) { }
		if(this.ttip_delay> 0) {
			clearTimeout(this.labelTrigger);
			this.labelTrigger= setTimeout("pie_act.showLabels();", this.ttip_delay);
		}
	},

	shieldCss : function(node) {
		node.style.display= "block";
		node.style.margin= "0px";
		node.style.padding= "0px";
		node.style.border= "none 0px";
		node.style.cssFloat= "none";
		node.style.direction= "ltr";
		
		try {
			node.style.color= "black";
			node.style.backgroundImage= "none";
			node.style.backgroundColor= "transparent";
		} catch(ex) { }
		//node.style.overflow= "hidden";
		//node.style.textIndent= "0px";
	},
	
	shieldTextCss : function(node) {
		node.style.font= "normal 8pt/20px Tahoma, sans-serif";
		node.style.verticalAlign= "baseline";
		node.style.textAlign= "left";
		node.style.textIndent= "0px";
	}
}
